template<>
struct __attribute__((packed)) guest_layout<UnionType> {
  using type = UnionType;
  type data;
};
template<>
struct guest_layout<const UnionType> : guest_layout<UnionType> {
  guest_layout& operator=(const guest_layout<UnionType>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<UnionType> {
  using type = UnionType;
  type data;

  host_layout(const guest_layout<UnionType>& from) :
    data { from.data } {
  }
};

inline guest_layout<UnionType> to_guest(const host_layout<UnionType>& from) {
  guest_layout<UnionType> ret;
  static_assert(sizeof(from) == sizeof(ret));
  memcpy(&ret, &from, sizeof(from));
  return ret;
}

void fex_apply_custom_repacking_entry(host_layout<UnionType>& source, const guest_layout<UnionType>& from) {
}
bool fex_apply_custom_repacking_exit(guest_layout<UnionType>& into, const host_layout<UnionType>& from) {
  return false;
}
template<> inline constexpr bool has_compatible_data_layout<UnionType> = true;
template<> inline constexpr bool has_compatible_data_layout<OpaqueType*> = true;
template<>
struct __attribute__((packed)) guest_layout<CustomRepackedType> {
  struct type {
    guest_layout<ReorderingType *> data;
    guest_layout<int32_t> custom_repack_invoked;
  };
  type data;
};
template<>
struct guest_layout<const CustomRepackedType> : guest_layout<CustomRepackedType> {
  guest_layout& operator=(const guest_layout<CustomRepackedType>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<CustomRepackedType> {
  using type = CustomRepackedType;
  type data;

  host_layout(const guest_layout<CustomRepackedType>& from) :
    data {
      .custom_repack_invoked = host_layout<int> { from.data.custom_repack_invoked }.data,
    } {
  }
};

inline guest_layout<CustomRepackedType> to_guest(const host_layout<CustomRepackedType>& from) {
  guest_layout<CustomRepackedType> ret { .data {
    .custom_repack_invoked = to_guest(to_host_layout(from.data.custom_repack_invoked)),
  } };
  return ret;
}

void fex_custom_repack_entry(host_layout<CustomRepackedType>& into, const guest_layout<CustomRepackedType>& from);
bool fex_custom_repack_exit(guest_layout<CustomRepackedType>& into, const host_layout<CustomRepackedType>& from);

void fex_apply_custom_repacking_entry(host_layout<CustomRepackedType>& source, const guest_layout<CustomRepackedType>& from) {
  fex_custom_repack_entry(source, from);
}
bool fex_apply_custom_repacking_exit(guest_layout<CustomRepackedType>& into, const host_layout<CustomRepackedType>& from) {
  return fex_custom_repack_exit(into, from);
}
template<> inline constexpr bool has_compatible_data_layout<CustomRepackedType> = false;
template<>
struct __attribute__((packed)) guest_layout<StructType> {
  using type = int32_t;
  type data;
};
template<>
struct __attribute__((packed)) guest_layout<TestStruct2> {
  struct type {
    guest_layout<const void *> Next;
    guest_layout<StructType> Type;
    guest_layout<int32_t> Data1;
  };
  type data;
};
template<>
struct guest_layout<const TestStruct2> : guest_layout<TestStruct2> {
  guest_layout& operator=(const guest_layout<TestStruct2>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<TestStruct2> {
  using type = TestStruct2;
  type data;

  host_layout(const guest_layout<TestStruct2>& from) :
    data {
      .Type = host_layout<StructType> { from.data.Type }.data,
      .Data1 = host_layout<int> { from.data.Data1 }.data,
    } {
  }
};

inline guest_layout<TestStruct2> to_guest(const host_layout<TestStruct2>& from) {
  guest_layout<TestStruct2> ret { .data {
    .Type = to_guest(to_host_layout(from.data.Type)),
    .Data1 = to_guest(to_host_layout(from.data.Data1)),
  } };
  return ret;
}

void fex_custom_repack_entry(host_layout<TestStruct2>& into, const guest_layout<TestStruct2>& from);
bool fex_custom_repack_exit(guest_layout<TestStruct2>& into, const host_layout<TestStruct2>& from);

void fex_apply_custom_repacking_entry(host_layout<TestStruct2>& source, const guest_layout<TestStruct2>& from) {
  fex_custom_repack_entry(source, from);
}
bool fex_apply_custom_repacking_exit(guest_layout<TestStruct2>& into, const host_layout<TestStruct2>& from) {
  return fex_custom_repack_exit(into, from);
}
template<> inline constexpr bool has_compatible_data_layout<TestStruct2> = false;
template<>
struct __attribute__((packed)) guest_layout<ReorderingType> {
  struct type {
    guest_layout<uint32_t> b;
    guest_layout<uint32_t> a;
  };
  type data;
};
template<>
struct guest_layout<const ReorderingType> : guest_layout<ReorderingType> {
  guest_layout& operator=(const guest_layout<ReorderingType>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<ReorderingType> {
  using type = ReorderingType;
  type data;

  host_layout(const guest_layout<ReorderingType>& from) :
    data {
      .a = host_layout<uint32_t> { from.data.a }.data,
      .b = host_layout<uint32_t> { from.data.b }.data,
    } {
  }
};

inline guest_layout<ReorderingType> to_guest(const host_layout<ReorderingType>& from) {
  guest_layout<ReorderingType> ret { .data {
    .b = to_guest(to_host_layout(from.data.b)),
    .a = to_guest(to_host_layout(from.data.a)),
  } };
  return ret;
}

void fex_apply_custom_repacking_entry(host_layout<ReorderingType>& source, const guest_layout<ReorderingType>& from) {
}
bool fex_apply_custom_repacking_exit(guest_layout<ReorderingType>& into, const host_layout<ReorderingType>& from) {
  return false;
}
template<> inline constexpr bool has_compatible_data_layout<ReorderingType> = false;
template<>
struct __attribute__((packed)) guest_layout<TestBaseStruct> {
  struct type {
    guest_layout<TestBaseStruct *> Next;
    guest_layout<StructType> Type;
  };
  type data;
};
template<>
struct guest_layout<const TestBaseStruct> : guest_layout<TestBaseStruct> {
  guest_layout& operator=(const guest_layout<TestBaseStruct>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<TestBaseStruct> {
  using type = TestBaseStruct;
  type data;

  host_layout(const guest_layout<TestBaseStruct>& from) :
    data {
      .Type = host_layout<StructType> { from.data.Type }.data,
    } {
  }
};

inline guest_layout<TestBaseStruct> to_guest(const host_layout<TestBaseStruct>& from) {
  guest_layout<TestBaseStruct> ret { .data {
    .Type = to_guest(to_host_layout(from.data.Type)),
  } };
  return ret;
}

void fex_custom_repack_entry(host_layout<TestBaseStruct>& into, const guest_layout<TestBaseStruct>& from);
bool fex_custom_repack_exit(guest_layout<TestBaseStruct>& into, const host_layout<TestBaseStruct>& from);

void fex_apply_custom_repacking_entry(host_layout<TestBaseStruct>& source, const guest_layout<TestBaseStruct>& from) {
  fex_custom_repack_entry(source, from);
}
bool fex_apply_custom_repacking_exit(guest_layout<TestBaseStruct>& into, const host_layout<TestBaseStruct>& from) {
  return fex_custom_repack_exit(into, from);
}
template<> inline constexpr bool has_compatible_data_layout<TestBaseStruct> = false;
template<>
struct __attribute__((packed)) guest_layout<DivType> {
  using type = uint32_t;
  type data;
};
template<>
struct __attribute__((packed)) guest_layout<TestStruct1> {
  struct type {
    guest_layout<const void *> Next;
    guest_layout<StructType> Type;
    guest_layout<uint8_t> Data2;
    guest_layout<uint8_t[3]> pad0;
    guest_layout<int32_t> Data1;
  };
  type data;
};
template<>
struct guest_layout<const TestStruct1> : guest_layout<TestStruct1> {
  guest_layout& operator=(const guest_layout<TestStruct1>& other) { memcpy(this, &other, sizeof(other)); return *this; }
};
template<>
struct host_layout<TestStruct1> {
  using type = TestStruct1;
  type data;

  host_layout(const guest_layout<TestStruct1>& from) :
    data {
      .Type = host_layout<StructType> { from.data.Type }.data,
      .Data2 = host_layout<uint8_t> { from.data.Data2 }.data,
      .Data1 = host_layout<int> { from.data.Data1 }.data,
    } {
      for (size_t i = 0; i < 3; ++i) {
        data.pad0[i] = host_layout<uint8_t[3]> { from.data.pad0 }.data[i];
      }
  }
};

inline guest_layout<TestStruct1> to_guest(const host_layout<TestStruct1>& from) {
  guest_layout<TestStruct1> ret { .data {
    .Type = to_guest(to_host_layout(from.data.Type)),
    .Data2 = to_guest(to_host_layout(from.data.Data2)),
    .Data1 = to_guest(to_host_layout(from.data.Data1)),
  } };
    for (size_t i = 0; i < 3; ++i) {
      ret.data.pad0.data[i] = to_guest(to_host_layout(from.data.pad0[i]));
    }
  return ret;
}

void fex_custom_repack_entry(host_layout<TestStruct1>& into, const guest_layout<TestStruct1>& from);
bool fex_custom_repack_exit(guest_layout<TestStruct1>& into, const host_layout<TestStruct1>& from);

void fex_apply_custom_repacking_entry(host_layout<TestStruct1>& source, const guest_layout<TestStruct1>& from) {
  fex_custom_repack_entry(source, from);
}
bool fex_apply_custom_repacking_exit(guest_layout<TestStruct1>& into, const host_layout<TestStruct1>& from) {
  return fex_custom_repack_exit(into, from);
}
template<> inline constexpr bool has_compatible_data_layout<TestStruct1> = false;
using fexldr_type_libfex_thunk_test_GetDoubledValue = auto (uint32_t a_0) -> uint32_t;
static fexldr_type_libfex_thunk_test_GetDoubledValue *fexldr_ptr_libfex_thunk_test_GetDoubledValue;
using fexldr_type_libfex_thunk_test_MakeOpaqueType = auto (uint32_t a_0) -> OpaqueType *;
static fexldr_type_libfex_thunk_test_MakeOpaqueType *fexldr_ptr_libfex_thunk_test_MakeOpaqueType;
using fexldr_type_libfex_thunk_test_ReadOpaqueTypeData = auto (OpaqueType * a_0) -> uint32_t;
static fexldr_type_libfex_thunk_test_ReadOpaqueTypeData *fexldr_ptr_libfex_thunk_test_ReadOpaqueTypeData;
using fexldr_type_libfex_thunk_test_DestroyOpaqueType = auto (OpaqueType * a_0) -> void;
static fexldr_type_libfex_thunk_test_DestroyOpaqueType *fexldr_ptr_libfex_thunk_test_DestroyOpaqueType;
using fexldr_type_libfex_thunk_test_MakeUnionType = auto (uint8_t a_0, uint8_t a_1, uint8_t a_2, uint8_t a_3) -> UnionType;
static fexldr_type_libfex_thunk_test_MakeUnionType *fexldr_ptr_libfex_thunk_test_MakeUnionType;
using fexldr_type_libfex_thunk_test_GetUnionTypeA = auto (UnionType * a_0) -> uint32_t;
static fexldr_type_libfex_thunk_test_GetUnionTypeA *fexldr_ptr_libfex_thunk_test_GetUnionTypeA;
using fexldr_type_libfex_thunk_test_MakeReorderingType = auto (uint32_t a_0, uint32_t a_1) -> ReorderingType;
static fexldr_type_libfex_thunk_test_MakeReorderingType *fexldr_ptr_libfex_thunk_test_MakeReorderingType;
using fexldr_type_libfex_thunk_test_GetReorderingTypeMember = auto (const ReorderingType * a_0, int a_1) -> uint32_t;
static fexldr_type_libfex_thunk_test_GetReorderingTypeMember *fexldr_ptr_libfex_thunk_test_GetReorderingTypeMember;
using fexldr_type_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking = auto (const ReorderingType * a_0, int a_1) -> uint32_t;
static fexldr_type_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking *fexldr_ptr_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking;
using fexldr_type_libfex_thunk_test_ModifyReorderingTypeMembers = auto (ReorderingType * a_0) -> void;
static fexldr_type_libfex_thunk_test_ModifyReorderingTypeMembers *fexldr_ptr_libfex_thunk_test_ModifyReorderingTypeMembers;
using fexldr_type_libfex_thunk_test_QueryOffsetOf = auto (ReorderingType * a_0, int a_1) -> uint32_t;
static fexldr_type_libfex_thunk_test_QueryOffsetOf *fexldr_ptr_libfex_thunk_test_QueryOffsetOf;
using fexldr_type_libfex_thunk_test_RanCustomRepack = auto (CustomRepackedType * a_0) -> int;
static fexldr_type_libfex_thunk_test_RanCustomRepack *fexldr_ptr_libfex_thunk_test_RanCustomRepack;
using fexldr_type_libfex_thunk_test_FunctionWithDivergentSignature = auto (DivType a_0, DivType a_1, DivType a_2, DivType a_3) -> int;
static fexldr_type_libfex_thunk_test_FunctionWithDivergentSignature *fexldr_ptr_libfex_thunk_test_FunctionWithDivergentSignature;
using fexldr_type_libfex_thunk_test_ReadData1 = auto (TestStruct1 * a_0, int a_1) -> int;
static fexldr_type_libfex_thunk_test_ReadData1 *fexldr_ptr_libfex_thunk_test_ReadData1;
extern "C" {
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_GetDoubledValue {
  guest_layout<uint32_t> a_0;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_GetDoubledValue(fexfn_packed_args_libfex_thunk_test_GetDoubledValue* args) {
  host_layout<uint32_t> a_0 { args->a_0 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexldr_ptr_libfex_thunk_test_GetDoubledValue(unwrap_host(a_0))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_MakeOpaqueType {
  guest_layout<uint32_t> a_0;
  guest_layout<OpaqueType *> rv;
};
static void fexfn_unpack_libfex_thunk_test_MakeOpaqueType(fexfn_packed_args_libfex_thunk_test_MakeOpaqueType* args) {
  host_layout<uint32_t> a_0 { args->a_0 };
  args->rv = to_guest(to_host_layout<OpaqueType *>(fexldr_ptr_libfex_thunk_test_MakeOpaqueType(unwrap_host(a_0))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_ReadOpaqueTypeData {
  guest_layout<OpaqueType *> a_0;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_ReadOpaqueTypeData(fexfn_packed_args_libfex_thunk_test_ReadOpaqueTypeData* args) {
  host_layout<OpaqueType *> a_0 { args->a_0 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexldr_ptr_libfex_thunk_test_ReadOpaqueTypeData(unwrap_host(a_0))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_DestroyOpaqueType {
  guest_layout<OpaqueType *> a_0;
};
static void fexfn_unpack_libfex_thunk_test_DestroyOpaqueType(fexfn_packed_args_libfex_thunk_test_DestroyOpaqueType* args) {
  host_layout<OpaqueType *> a_0 { args->a_0 };
fexldr_ptr_libfex_thunk_test_DestroyOpaqueType(unwrap_host(a_0));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_MakeUnionType {
  guest_layout<uint8_t> a_0;
  guest_layout<uint8_t> a_1;
  guest_layout<uint8_t> a_2;
  guest_layout<uint8_t> a_3;
  guest_layout<UnionType> rv;
};
static void fexfn_unpack_libfex_thunk_test_MakeUnionType(fexfn_packed_args_libfex_thunk_test_MakeUnionType* args) {
  host_layout<uint8_t> a_0 { args->a_0 };
  host_layout<uint8_t> a_1 { args->a_1 };
  host_layout<uint8_t> a_2 { args->a_2 };
  host_layout<uint8_t> a_3 { args->a_3 };
  args->rv = to_guest(to_host_layout<UnionType>(fexldr_ptr_libfex_thunk_test_MakeUnionType(unwrap_host(a_0), unwrap_host(a_1), unwrap_host(a_2), unwrap_host(a_3))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_GetUnionTypeA {
  guest_layout<UnionType *> a_0;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_GetUnionTypeA(fexfn_packed_args_libfex_thunk_test_GetUnionTypeA* args) {
  host_layout<UnionType *> a_0 { args->a_0 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexldr_ptr_libfex_thunk_test_GetUnionTypeA(unwrap_host(a_0))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_MakeReorderingType {
  guest_layout<uint32_t> a_0;
  guest_layout<uint32_t> a_1;
  guest_layout<ReorderingType> rv;
};
static void fexfn_unpack_libfex_thunk_test_MakeReorderingType(fexfn_packed_args_libfex_thunk_test_MakeReorderingType* args) {
  host_layout<uint32_t> a_0 { args->a_0 };
  host_layout<uint32_t> a_1 { args->a_1 };
  args->rv = to_guest(to_host_layout<ReorderingType>(fexldr_ptr_libfex_thunk_test_MakeReorderingType(unwrap_host(a_0), unwrap_host(a_1))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_GetReorderingTypeMember {
  guest_layout<const ReorderingType *> a_0;
  guest_layout<int32_t> a_1;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_GetReorderingTypeMember(fexfn_packed_args_libfex_thunk_test_GetReorderingTypeMember* args) {
  auto a_0 = make_repack_wrapper<ReorderingType *>(args->a_0);
  host_layout<int> a_1 { args->a_1 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexldr_ptr_libfex_thunk_test_GetReorderingTypeMember(unwrap_host(a_0), unwrap_host(a_1))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking {
  guest_layout<const ReorderingType *> a_0;
  guest_layout<int32_t> a_1;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking(fexfn_packed_args_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking* args) {
  host_layout<const struct ReorderingType *> a_0 { args->a_0 };
  host_layout<int> a_1 { args->a_1 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexldr_ptr_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking(unwrap_host(a_0), unwrap_host(a_1))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_ModifyReorderingTypeMembers {
  guest_layout<ReorderingType *> a_0;
};
static void fexfn_unpack_libfex_thunk_test_ModifyReorderingTypeMembers(fexfn_packed_args_libfex_thunk_test_ModifyReorderingTypeMembers* args) {
  auto a_0 = make_repack_wrapper<ReorderingType *>(args->a_0);
fexldr_ptr_libfex_thunk_test_ModifyReorderingTypeMembers(unwrap_host(a_0));
}
static auto fexfn_impl_libfex_thunk_test_QueryOffsetOf(guest_layout<ReorderingType *> a_0, int a_1) -> uint32_t;
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_QueryOffsetOf {
  guest_layout<ReorderingType *> a_0;
  guest_layout<int32_t> a_1;
  guest_layout<uint32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_QueryOffsetOf(fexfn_packed_args_libfex_thunk_test_QueryOffsetOf* args) {
  host_layout<int> a_1 { args->a_1 };
  args->rv = to_guest(to_host_layout<uint32_t>(fexfn_impl_libfex_thunk_test_QueryOffsetOf(args->a_0, unwrap_host(a_1))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_RanCustomRepack {
  guest_layout<CustomRepackedType *> a_0;
  guest_layout<int32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_RanCustomRepack(fexfn_packed_args_libfex_thunk_test_RanCustomRepack* args) {
  auto a_0 = make_repack_wrapper<CustomRepackedType *>(args->a_0);
  args->rv = to_guest(to_host_layout<int>(fexldr_ptr_libfex_thunk_test_RanCustomRepack(unwrap_host(a_0))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_FunctionWithDivergentSignature {
  guest_layout<DivType> a_0;
  guest_layout<DivType> a_1;
  guest_layout<DivType> a_2;
  guest_layout<DivType> a_3;
  guest_layout<int32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_FunctionWithDivergentSignature(fexfn_packed_args_libfex_thunk_test_FunctionWithDivergentSignature* args) {
  host_layout<DivType> a_0 { args->a_0 };
  host_layout<DivType> a_1 { args->a_1 };
  host_layout<DivType> a_2 { args->a_2 };
  host_layout<DivType> a_3 { args->a_3 };
  args->rv = to_guest(to_host_layout<int>(fexldr_ptr_libfex_thunk_test_FunctionWithDivergentSignature(unwrap_host(a_0), unwrap_host(a_1), unwrap_host(a_2), unwrap_host(a_3))));
}
struct __attribute__((packed)) fexfn_packed_args_libfex_thunk_test_ReadData1 {
  guest_layout<TestStruct1 *> a_0;
  guest_layout<int32_t> a_1;
  guest_layout<int32_t> rv;
};
static void fexfn_unpack_libfex_thunk_test_ReadData1(fexfn_packed_args_libfex_thunk_test_ReadData1* args) {
  auto a_0 = make_repack_wrapper<TestStruct1 *>(args->a_0);
  host_layout<int> a_1 { args->a_1 };
  args->rv = to_guest(to_host_layout<int>(fexldr_ptr_libfex_thunk_test_ReadData1(unwrap_host(a_0), unwrap_host(a_1))));
}
}
static ExportEntry exports[] = {
  {(uint8_t*)"\x3d\xf2\xa0\x4f\xcd\x99\xdb\x2b\x8b\x74\x79\x11\x7c\x08\xcc\xfb\x0b\xf1\x7c\x43\xec\xea\xe6\x56\x43\x27\xa9\x26\x69\x93\x0d\xd9", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_GetDoubledValue}, // libfex_thunk_test:GetDoubledValue
  {(uint8_t*)"\x45\x12\x28\xe0\x8b\x6c\x93\x81\x30\x5d\xa9\x85\xec\x1e\x5c\x23\xc8\x3c\x61\xb5\x09\x0f\xc8\x80\x7e\x2a\xb7\x8d\x52\xec\xe3\xdb", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_MakeOpaqueType}, // libfex_thunk_test:MakeOpaqueType
  {(uint8_t*)"\x11\x88\x54\xe8\x05\xc6\xf7\xd1\x44\x97\xc8\xa7\x2a\xf8\xe3\xbd\xee\xfb\x2c\xa1\x4c\x32\x61\x88\xed\xc0\x2c\x93\xa0\xf0\x90\x5b", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_ReadOpaqueTypeData}, // libfex_thunk_test:ReadOpaqueTypeData
  {(uint8_t*)"\xa9\x32\x5f\x89\xd6\x5f\xc0\x53\x5d\xed\xd2\x91\x90\xd0\x7b\x93\xe7\x84\xe8\x92\x3c\xf4\x29\xd9\xbb\xf6\xee\x93\xa2\x28\x48\x24", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_DestroyOpaqueType}, // libfex_thunk_test:DestroyOpaqueType
  {(uint8_t*)"\xe3\xad\x8c\x7b\xa6\x43\x49\x61\x7d\xcf\x86\x81\x2b\x96\x28\x2f\xdb\x4d\x9e\x48\xad\xf1\x6b\x2c\x14\xb9\xec\xe6\x54\x06\x45\xe4", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_MakeUnionType}, // libfex_thunk_test:MakeUnionType
  {(uint8_t*)"\xed\xb8\x48\xc7\x14\x80\x2b\x41\x38\x7b\x1e\xc1\x4d\xca\x74\x70\x70\xdc\xd9\x7d\x69\xd2\x5b\x05\x8a\x56\xad\x98\xba\x62\xb0\x4b", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_GetUnionTypeA}, // libfex_thunk_test:GetUnionTypeA
  {(uint8_t*)"\xed\x91\x8f\x3e\x6e\x59\x6e\xeb\x97\x8c\x82\x26\xee\xca\x07\xbf\x3f\x3b\xb5\x60\x89\x17\xd2\xdd\x96\x21\xcd\x01\x7b\x8c\x6e\x8e", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_MakeReorderingType}, // libfex_thunk_test:MakeReorderingType
  {(uint8_t*)"\x22\x79\xa4\xe0\xce\xba\x70\x93\x36\x14\xa4\xc9\x29\x6a\xe6\xd6\x55\x75\xd6\x6d\xab\x47\x5c\x13\x92\x22\x48\x8c\x0a\xb2\x27\x83", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_GetReorderingTypeMember}, // libfex_thunk_test:GetReorderingTypeMember
  {(uint8_t*)"\xcd\xc6\xea\x64\x84\x48\x90\x7d\x83\x67\xad\xa3\x5f\x35\xf4\x9f\xcc\x1b\xad\x36\xcc\xf4\x4c\xf7\xf2\xc1\xf8\xcd\x02\xee\xbe\x20", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking}, // libfex_thunk_test:GetReorderingTypeMemberWithoutRepacking
  {(uint8_t*)"\x87\x80\x68\xa5\x4a\xb7\xf4\x9c\x38\x2c\x88\x7d\xfe\x53\xb0\x8f\x72\x2a\xa6\xb7\xd3\x73\x63\xe7\x09\xd5\xc0\xf0\x86\x6e\xd8\xe0", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_ModifyReorderingTypeMembers}, // libfex_thunk_test:ModifyReorderingTypeMembers
  {(uint8_t*)"\x1a\xf0\x22\xb8\xef\xc3\x16\x91\xbd\x81\xdf\x7e\xf1\xc7\x77\x12\xfd\x9c\x86\x82\xdb\x4f\xb2\x79\x7a\x16\xe9\x6f\x68\x4a\x0c\x23", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_QueryOffsetOf}, // libfex_thunk_test:QueryOffsetOf
  {(uint8_t*)"\xaf\x96\x72\xd5\xf1\x8a\xed\x26\x4e\x52\x3a\x61\x59\xb1\xc5\x4a\xfe\xbb\xf7\xa6\x80\x77\x7e\x64\x26\x53\x41\xb8\x9e\x94\x73\x14", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_RanCustomRepack}, // libfex_thunk_test:RanCustomRepack
  {(uint8_t*)"\x14\x0c\x50\x9a\x69\xb7\x71\xdb\xce\x9e\xd7\x97\x0a\xce\x81\xa2\xa0\x9b\xe6\x72\xc1\xbe\xec\x01\x00\xed\x4d\x2b\x84\x05\x52\x3f", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_FunctionWithDivergentSignature}, // libfex_thunk_test:FunctionWithDivergentSignature
  {(uint8_t*)"\x72\x12\xea\x6b\x06\x99\x68\x70\x5d\x04\x8f\xec\x5d\xfb\x98\xbd\xa3\x2c\x0a\xe9\xb9\x97\xf7\x22\x3d\x9c\x02\x4b\x23\x4f\xe8\x9a", (void(*)(void *))&fexfn_unpack_libfex_thunk_test_ReadData1}, // libfex_thunk_test:ReadData1
  { nullptr, nullptr }
};
static void* fexldr_ptr_libfex_thunk_test_so;
extern "C" bool fexldr_init_libfex_thunk_test() {
  fexldr_ptr_libfex_thunk_test_so = dlopen("libfex_thunk_test.so", RTLD_GLOBAL | RTLD_LAZY);
  if (!fexldr_ptr_libfex_thunk_test_so) { return false; }

  (void*&)fexldr_ptr_libfex_thunk_test_GetDoubledValue = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "GetDoubledValue");
  (void*&)fexldr_ptr_libfex_thunk_test_MakeOpaqueType = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "MakeOpaqueType");
  (void*&)fexldr_ptr_libfex_thunk_test_ReadOpaqueTypeData = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "ReadOpaqueTypeData");
  (void*&)fexldr_ptr_libfex_thunk_test_DestroyOpaqueType = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "DestroyOpaqueType");
  (void*&)fexldr_ptr_libfex_thunk_test_MakeUnionType = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "MakeUnionType");
  (void*&)fexldr_ptr_libfex_thunk_test_GetUnionTypeA = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "GetUnionTypeA");
  (void*&)fexldr_ptr_libfex_thunk_test_MakeReorderingType = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "MakeReorderingType");
  (void*&)fexldr_ptr_libfex_thunk_test_GetReorderingTypeMember = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "GetReorderingTypeMember");
  (void*&)fexldr_ptr_libfex_thunk_test_GetReorderingTypeMemberWithoutRepacking = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "GetReorderingTypeMemberWithoutRepacking");
  (void*&)fexldr_ptr_libfex_thunk_test_ModifyReorderingTypeMembers = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "ModifyReorderingTypeMembers");
  (void*&)fexldr_ptr_libfex_thunk_test_QueryOffsetOf = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "QueryOffsetOf");
  (void*&)fexldr_ptr_libfex_thunk_test_RanCustomRepack = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "RanCustomRepack");
  (void*&)fexldr_ptr_libfex_thunk_test_FunctionWithDivergentSignature = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "FunctionWithDivergentSignature");
  (void*&)fexldr_ptr_libfex_thunk_test_ReadData1 = dlsym_default(fexldr_ptr_libfex_thunk_test_so, "ReadData1");
  return true;
}
